BPBackend

Enum BPBackend 

pub enum BPBackend {
    Dynamic,
    Prealloc {
        capacity: usize,
    },
}
Available allocation backends for BufPool

Variants



Dynamic

All slots are dynamically constructed at runtime, avoids waiting for slot availablity under high contention at cost of runtime allocations

When to use
  • burst workloads
  • low-contention code paths
  • requests that may exceed the configured pool capacity


Prealloc

Uses a pre-allocated freelist w/ the given capacity

All the chunks are allocated upfront, avoiding runtime allocations, while providing lower and more predicatble latency compared to BPBackend::Dynamic

When to use
  • hot IO paths
  • write pipelines
  • storage engines
  • workloads where allocation latency must remain stable

If all buffers are currently in use, BufPool::allocate blocks until another Allocation is dropped and buffers return to the pool

Fallback

For BPBackend::Prealloc backend, f n exceeds the pool capacity, the allocation is performed using the BPBackend::Dynamic

Fields

capacity: usize

Number of chunks to pre-allocate in memory