PoolPtr

Struct PoolPtr 

pub struct PoolPtr {
    pub ptr: *mut u8,
}
Pointer to buffer allocated by BPool::allocate

Fields

ptr: *mut u8

Raw pointer to the start of a buffer owned by the pool, where the valid memory range is [ptr, ptr + buf_size)