pub struct FMCfg {
pub initial_count: usize,
pub flush_duration: Duration,
}Expand description
Config for FrozenMMap
Fields
initial_count: usizeNumber of slots to pre-allocate when FrozenMMap is initialized
Each slot has size of FrozenMMap::<T>::SLOT_SIZE, where initial file length will
be chunk_size * initial_count (bytes)
flush_duration: DurationTime interval used by flusher tx, to batch write ops into a durable window and sync them together, where all write ops in certain time interval falls into a single durable window