WritePipeCfg

Struct WritePipeCfg 

pub struct WritePipeCfg {
    pub module_id: u8,
    pub flush_duration: Duration,
}
All the available configurations for WritePipe

Example

use frozen_core::wpipe::WritePipeCfg;

let cfg = WritePipeCfg {
    module_id: 2,
    flush_duration: std::time::Duration::from_millis(0x0A),
};

assert_ne!(cfg.module_id, 0);
assert_ne!(cfg.flush_duration.as_millis(), 0);

Fields

module_id: u8

Identifier used for error propagation by [frozen_core::error::FrozenError]

flush_duration: Duration

Time interval used by the background thread to perform hard sync for all the write operations submitted in the last durability window