Crate frozen_core

Crate frozen_core 

Latest Version License Tests

FrozenCore

Custom implementations and core utilities for frozen-lab crates.

Usage

Add following to your Cargo.toml,

[dependencies]
frozen-core = { version = "0.0.23", default-features = true }

[!NOTE] Current frozen-core requires Rust 1.86 or later.

[!TIP] All the features are enabled by default. To disable, set default-features to false.

Feature Flags

Notes

[!IMPORTANT] frozen-core is primarily created for frozen-lab projects. External use is discouraged, but not prohibited, given you assume all the risks.

This project is licensed under the Apache-2.0 and MIT License. See the LICENSE-APACHE and LICENSE-MIT file for more details.

Contributions are welcome! Please feel free to submit a PR or open an issue if you have any feedback or suggestions.

[!NOTE] frozen-core contains next to naught AI-generated code. Therefore, any catastrophic bugs or fatal crashes encountered are results of pure & unadulterated skill issues

Modules

bufpool
A low-latency memory-budgeted buffer pool to manage fixed-sized buffer allocations
crc32
Implementation of CRC32C (Castagnoli polynomial) to compute a 32-bit cyclic redundancy check (CRC) using Castagnoli polynomial, intended for data integrity verification for torn writes and curruption detection
error
Utilities for error propagation used across [frozen_core]
ffile
Custom implementation of std::fs::File
fmmap
Custom implementation of mmap(2)
hints
Implementation of branch preditor hints
mpscq
A low-latency implementation of MPSC (multi-producer single-consumer) queue
utils
Common utilities used in [frozen_core]
wpipe
A low latency asynchronous write pipeline for buffer based storage