AckTicket

Struct AckTicket 

pub struct AckTicket { /* private fields */ }
Durability handle associated with the write operation

Epoch

Every ticket is assigned a monotonically increasing epoch to moniter durability

Durability Guarantee

If wanted, the ticket could be awaited to poll till the epoch becomes durable.

Once a await on ticket is completed successfully, all writes assigned to earlier epochs are also guaranteed to be durable.

NOTE: Using await is optional. Callers that only require fire-and-forget semantics may simply discard the returned ticket.

Implementations

Source

impl AckTicket

pub const fn new(epoch: TEpoch, completion: Arc<Completion>) -> Self

Construct a new AckTicket for a write operation

Example
use frozen_core::ack::{AckTicket, Completion};
use std::sync::Arc;

let completion = Arc::new(Completion::default());
let ticket = AckTicket::new(1, completion);

assert_eq!(ticket.epoch(), 1);
pub const fn epoch(&self) -> TEpoch

Read assigned durability epoch for the AckTicket

Example
use frozen_core::ack::{AckTicket, Completion};
use std::sync::Arc;

let completion = Arc::new(Completion::default());
let ticket = AckTicket::new(0x4C, completion);

assert_eq!(ticket.epoch(), 0x4C);