pub struct AckTicket { /* private fields */ }Expand description
Durability handle associated with the write operation
Epoch
Every ticket is assigned a monotonically increasing epoch to moniter durability
Durability Guarantee
If wanted, the ticket could be awaited to poll till the epoch becomes durable.
Once a await on ticket is completed successfully, all writes assigned to earlier epochs are also guaranteed to be durable.
NOTE: Using await is optional. Callers that only require fire-and-forget semantics may
simply discard the returned ticket.
Implementations