pub struct BufPoolCfg {
pub buffer_size: BufferSize,
pub max_memory: usize,
}Expand description
All the available configrations for BufPool
Example
use frozen_core::{bufpool::BufPoolCfg, utils::BufferSize};
const BUF_SIZE: BufferSize = BufferSize::S64;
let cfg = BufPoolCfg {
buffer_size: BUF_SIZE,
max_memory: BUF_SIZE as usize * 0x1000,
};
assert_ne!(cfg.max_memory, 0);
assert!(cfg.max_memory > cfg.buffer_size.bytes());Fields
buffer_size: BufferSizeSize (in bytes) of an indivdual buffer unit allocated
max_memory: usizeMaximum allowed memory (in bytes) to be simultaneosuly allocated by BufPool
IMPORTANT: When trying to allocate more memory then BufPoolCfg::max_memory via
BufPool::allocate, a deadlock will happen due to memory budgeting in place. The caller
must make sure the max_meory is high enough to avoid this scenerio.
NOTE: To avoid backpressure, set the max_memory to an arbitrary large value. This
would not have any direct impact on performance or resource usage, and will avoid
backpressure under heavy workload.