FrozenFile

Struct FrozenFile 

pub struct FrozenFile { /* private fields */ }
Custom implementation of std::fs::File

Example

use frozen_core::ffile::{FrozenFile, FFCfg};

let dir = tempfile::tempdir().unwrap();
let path = dir.path().join("tmp_frozen_file");

let cfg = FFCfg {
    mid: 0x00,
    chunk_size: 0x10,
    path: path.to_path_buf(),
    initial_chunk_amount: 0x0A,
};

let file = FrozenFile::new(cfg.clone()).unwrap();
assert_eq!(file.length().unwrap(), 0x10 * 0x0A);

let mut data = vec![1u8; 0x10];
assert!(file.pwrite(data.as_mut_ptr(), 0).is_ok());
assert!(file.sync().is_ok());

let mut buf = vec![0u8; data.len()];
assert!(file.pread(buf.as_mut_ptr(), 0).is_ok());
assert_eq!(buf, data);

assert!(FrozenFile::new(cfg.clone()).is_err());

assert!(file.delete().is_ok());
assert!(!path.exists());

drop(file);
assert!(FrozenFile::new(cfg).is_ok());

Implementations

Source

impl FrozenFile

pub fn cfg(&self) -> &FFCfg

Fetch config used for FrozenFile

pub fn length(&self) -> FrozenRes<usize>

Read current length of FrozenFile

pub fn fd(&self) -> i32

Get file descriptor for FrozenFile

pub fn exists(&self) -> FrozenRes<bool>

Check if FrozenFile exists on the fs

pub fn new(cfg: FFCfg) -> FrozenRes<Self>

Create a new or open an existing FrozenFile

FFCfg

All configs for FrozenFile are stored in FFCfg

Important

The cfg must not change any of its properties for the entire life of FrozenFile, one must use config stores like Rta to store config

Multiple Instances

We acquire an exclusive lock for the entire file, this protects against operating with multiple simultenious instance of FrozenFile, when trying to call FrozenFile::new when already called, FFileErr::Lck error will be thrown

Example
use frozen_core::ffile::{FrozenFile, FFCfg};

let dir = tempfile::tempdir().unwrap();
let path = dir.path().join("tmp_frozen_file");

let cfg = FFCfg {
    mid: 0x00,
    chunk_size: 0x10,
    path: path.to_path_buf(),
    initial_chunk_amount: 0x0A,
};

let file = FrozenFile::new(cfg).unwrap();
assert_eq!(file.length().unwrap(), 0x10 * 0x0A);
pub fn grow(&self, count: usize) -> FrozenRes<()>

Grow file size of FrozenFile by given count of chunks

Example
use frozen_core::ffile::{FrozenFile, FFCfg};

let dir = tempfile::tempdir().unwrap();
let path = dir.path().join("tmp_frozen_file");

let cfg = FFCfg {
    mid: 0x00,
    chunk_size: 0x10,
    path: path.to_path_buf(),
    initial_chunk_amount: 0x0A,
};

let file = FrozenFile::new(cfg).unwrap();
assert_eq!(file.length().unwrap(), 0x10 * 0x0A);

file.grow(0x20).unwrap();
assert_eq!(file.length().unwrap(), 0x10 * (0x0A + 0x20));
pub fn sync(&self) -> FrozenRes<()>

Syncs in-mem data on the storage device

pub fn sync_range(&self, index: usize, count: usize) -> FrozenRes<()>

Initiates writeback (best-effort) of dirty pages in the specified range

pub fn delete(&self) -> FrozenRes<()>

Delete FrozenFile from fs

Example
use frozen_core::ffile::{FrozenFile, FFCfg};

let dir = tempfile::tempdir().unwrap();
let path = dir.path().join("tmp_frozen_file");

let cfg = FFCfg {
    mid: 0x00,
    chunk_size: 0x10,
    path: path.to_path_buf(),
    initial_chunk_amount: 0x0A,
};

let file = FrozenFile::new(cfg).unwrap();
assert!(file.exists().unwrap());

file.delete().unwrap();
assert!(!file.exists().unwrap());
pub fn pread(&self, buf: *mut u8, index: usize) -> FrozenRes<()>

Read a single chunk at given index w/ pread syscall

pub fn pwrite(&self, buf: *mut u8, index: usize) -> FrozenRes<()>

Write a single chunk at given index w/ pwrite syscall

pub fn preadv(&self, bufs: &[*mut u8], index: usize) -> FrozenRes<()>

Read multiple chunks starting from given index till bufs.len() w/ preadv syscall

pub fn pwritev(&self, bufs: &[*mut u8], index: usize) -> FrozenRes<()>

Write multiple chunks starting from given index till bufs.len() w/ pwritev syscall