MPSCQueue

Struct MPSCQueue 

pub struct MPSCQueue<T> { /* private fields */ }
A low-latency implementation of MPSC (multi-producer single-consumer) queue

Example

use frozen_core::mpscq::MPSCQueue;

let queue = MPSCQueue::<usize>::default();

queue.push(0x100);
queue.push(0x200);

let batch: Vec<usize> = queue.drain();

assert_eq!(batch.len(), 2);
assert_eq!(batch, vec![0x200, 0x100]);

Implementations

Source

impl<T> MPSCQueue<T>

pub fn is_empty(&self) -> bool

Checks if the MPSCQueue is currently empty

Example
use frozen_core::mpscq::MPSCQueue;

let queue = MPSCQueue::<usize>::default();
assert!(queue.is_empty());
pub fn push(&self, value: T)

Push an entry into the MPSCQueue

Example
use frozen_core::mpscq::MPSCQueue;

let queue = MPSCQueue::<usize>::default();
queue.push(0x0A);

let batch = queue.drain();

assert_eq!(batch.len(), 1);
assert_eq!(batch, vec![0x0A]);
pub fn drain(&self) -> Vec<T>

Drain the current batch of items from the MPSCQueue

Ordering

The queue internally uses a linked list like structure for storing T, therefore entries are stored in LIFO order.

Multiple Consumers

By design MPSCQueue::drain is thread safe, but draining from multiple threads is semantically incorrect. In the MPSC model, there should only be a single consumer, while there can be many producers.

Example
use frozen_core::mpscq::MPSCQueue;

let queue = MPSCQueue::<u8>::default();

queue.push(0x0A);
queue.push(0x0B);
queue.push(0x0C);

let batch = queue.drain();

assert_eq!(batch.len(), 3);
assert_eq!(batch, vec![0x0C, 0x0B, 0x0A]);