MPSCQueue

Struct MPSCQueue 

pub struct MPSCQueue<T> { /* private fields */ }
A lock-free multi-producer single-consumer queue

Example

let queue = frozen_core::mpscq::MPSCQueue::<usize>::default();

queue.push(1usize);
queue.push(2usize);

let batch = queue.drain();
assert_eq!(batch, vec![2usize, 1usize]);

drop(batch); // values is dropped here

Implementations

Source

impl<T> MPSCQueue<T>

pub fn is_empty(&self) -> bool

Check if the MPSCQueue is currently empty

pub fn push(&self, value: T)

Push an entry into the MPSCQueue

Ordering

The queue internally uses a linked list, therefore entries are stored in last-in-first-out (LIFO) order

Example
let queue = frozen_core::mpscq::MPSCQueue::<usize>::default();
queue.push(1usize);

let batch = queue.drain();
assert_eq!(batch.len(), 1);

drop(batch); // values is dropped here
pub fn drain(&self) -> Vec<T>

Drain a batch of entries from MPSCQueue

Ordering

The queue internally behaves like a stack, therefore entries are returned in last-in-first-out (LIFO) order when drained

Example
let queue = frozen_core::mpscq::MPSCQueue::<usize>::default();

queue.push(1usize);
queue.push(2usize);
queue.push(3usize);

let batch = queue.drain();

assert_eq!(batch.len(), 3);
assert_eq!(batch, vec![3usize, 2usize, 1usize]);

drop(batch); // values is dropped here